@@ -24,6 +24,7 @@ static volatile unsigned active_targets = 0;
 static volatile unsigned published_routes = 0;
 static volatile unsigned allocated_routes = 0;
 static volatile unsigned allocated_targets = 0;
+static volatile unsigned allocated_entries = 0;
 static unsigned mapped_entries = 999;
 
 static unsigned keysize = 177;
@@ -37,6 +38,7 @@ static LinkedObject *freetargets = NULL;
 static LinkedObject **keys = NULL;
 static condlock_t locking;
 static stats *statmap = NULL;
+static LinkedObject *freelist = NULL;
 
 registry registry::reg;
 
@@ -102,7 +104,7 @@ void registry::route::operator delete(void *obj)
 }
 
 registry::registry() :
-service::callback(0), mapped_reuse<MappedRegistry>()
+service::callback(0), mapped_array<MappedRegistry>()
 {
 	realm = (char *)"Local Telephony";
 	digest = (char *)"MD5";
@@ -198,7 +200,7 @@ void registry::start(service *cfg)
 	assert(cfg != NULL);
 
 	process::errlog(DEBUG1, "registry starting; mapping %d entries", mapped_entries);
-	MappedReuse::create(REGISTRY_MAP, mapped_entries);
+	create(REGISTRY_MAP, mapped_entries);
 	if(!reg)
 		process::errlog(FAILURE, "registry could not be mapped");
 	initialize();
@@ -242,10 +244,11 @@ void registry::snapshot(FILE *fp)
 	fprintf(fp, "  published routes:  %d\n", published_routes);
 	fprintf(fp, "  allocated routes:  %d\n", allocated_routes);
 	fprintf(fp, "  allocated targets: %d\n", allocated_targets);
+	fprintf(fp, "  allocated entries: %d\n", allocated_entries);
 
 	while(regcount < mapped_entries) {
 		time(&now);
-		rr = static_cast<mapped*>(reg.pos(regcount++));
+		rr = static_cast<mapped*>(reg(regcount++));
 		if(rr->type == MappedRegistry::TEMPORARY) {
 			fprintf(fp, "  temp %s; use=%d\n", rr->userid, rr->inuse);
 		}
@@ -315,6 +318,27 @@ void registry::snapshot(FILE *fp)
 	locking.release();
 } 
 
+void registry::clear(mapped *rr)
+{
+	assert(rr != NULL);
+	
+	rr->userid[0] = 0;
+	rr->display[0] = 0;
+	rr->remote[0] = 0;
+	rr->status = MappedRegistry::OFFLINE;
+	rr->type = MappedRegistry::EXPIRED;
+	rr->hidden = false;
+	rr->rid = -1;
+	rr->ext = 0;
+	rr->count = 0;
+	rr->inuse = 0;
+	rr->created = 0;
+	rr->expires = 0;
+	// this one is safe to clear...
+	memset(&rr->profile, 0, sizeof(profile_t));
+	rr->internal.published = rr->internal.targets = rr->internal.routes = NULL;
+}
+	
 bool registry::remove(const char *id)
 {
 	assert(id != NULL && *id != 0);
@@ -394,8 +418,8 @@ void registry::expire(mapped *rr)
 	rr->status = MappedRegistry::OFFLINE;
 	rr->type = MappedRegistry::EXPIRED;
 	rr->rid = -1;
-	rp->delist(&keys[path]);
-	reg.removeLocked(rr);
+	rr->delist(&keys[path]);
+	rr->enlist(&freelist);
 }
 
 void registry::cleanup(time_t period)
@@ -406,14 +430,14 @@ void registry::cleanup(time_t period)
 
 	while(regcount < mapped_entries) {
 		time(&now);
-		rr = static_cast<mapped*>(reg.pos(regcount++));
+		rr = static_cast<mapped*>(reg(regcount++));
 		locking.modify();
 		if(rr->type != MappedRegistry::EXPIRED && rr->expires && rr->expires + period < now && !rr->inuse) {
 			server::expire(rr);
 			expire(rr);
 		}
-		else if(!rr->inuse && rr->type == MappedRegistry::EXPIRED && !rr->status == MappedRegistry::OFFLINE)
-			server::expire(rr);
+		else if(!rr->inuse && rr->type == MappedRegistry::EXPIRED && rr->status != MappedRegistry::OFFLINE) 
+			expire(rr);
 		locking.commit();
 		Thread::yield();
 	}
@@ -493,14 +517,18 @@ registry::mapped *registry::invite(const char *id, stats::stat_t stat)
 	}
 
 	locking.exclusive();
-	rr = static_cast<mapped*>(reg.getLocked());
-
-	if(!rr) {
+	if(freelist) {
+		rr = (mapped *)freelist;
+		freelist = rr->getNext();
+	}
+	else if(allocated_entries < mapped_entries)
+		rr = (mapped *)reg(allocated_entries++);
+	else {
 		locking.commit();
 		return NULL;
 	}
 
-	memset(rr, 0, sizeof(mapped));
+	clear(rr);
 	rr->type = MappedRegistry::TEMPORARY;
 	rr->expires = 0;
 	rr->created = 0;
@@ -517,7 +545,7 @@ registry::mapped *registry::invite(const char *id, stats::stat_t stat)
 	return rr;
 }
 
-registry::mapped *registry::create(const char *id)
+registry::mapped *registry::allocate(const char *id)
 {
 	assert(id != NULL && *id != 0);
 
@@ -541,9 +569,13 @@ registry::mapped *registry::create(const char *id)
 	if(rr) 
 		listed = true;
 	else {
-		rr = static_cast<mapped*>(reg.getLocked());
+		if(freelist) {
+			rr = (mapped *)freelist;
+			freelist = rr->getNext();
+		} else if(allocated_entries < mapped_entries)
+			rr = (mapped *)reg(allocated_entries++);
 		if(rr)
-			memset(rr, 0, sizeof(mapped));
+			clear(rr);
 	}
 	if(!rr) {
 		locking.commit();
@@ -584,7 +616,7 @@ registry::mapped *registry::create(const char *id)
 			listed = false;
 		}
 		if(!listed)
-			reg.removeLocked(rr);
+			rr->enlist(&freelist);
 		locking.commit();
 		return NULL;
 	}
