@@ -311,11 +311,13 @@ void server::confirm(const char *user)
 	} 
 #else
 	if(user) {
-		dirpath = DEFAULT_VARPATH "/lib/sipwitch";
+		dirpath = DEFAULT_CFGPATH "/sipwitch.d";
 		fsys::open(dir, dirpath, fsys::ACCESS_DIRECTORY);
-		if(!dir)
-			dirpath = DEFAULT_CFGPATH "/sipwitch.d";
-	}		
+		if(!is(dir) && fsys::isdir(DEFAULT_VARPATH "/lib/sipwitch"))
+			dirpath = DEFAULT_VARPATH "/lib/sipwitch";
+		else if(!is(dir))
+			dirpath = ".";
+	}	
 	if(!is(dir))
 		fsys::open(dir, dirpath, fsys::ACCESS_DIRECTORY);
 #endif
@@ -324,7 +326,7 @@ void server::confirm(const char *user)
 	process::errlog(DEBUG1, "scanning config from %s", dirpath);
 	while(is(dir) && fsys::read(dir, filename, sizeof(filename)) > 0) {
 		ext = strrchr(filename, '.');
-		if(!ext || stricmp(ext, ".xml"))
+		if(ext && stricmp(ext, ".xml"))
 			continue;
 		snprintf(buf, sizeof(buf), "%s/%s", dirpath, filename);
 		fp = fopen(buf, "r");
