@@ -54,6 +54,15 @@ static void system_methods(void);
 static void system_help(void);
 static void system_signature(void);
 static void system_status(void);
+static void server_status(void);
+static void server_control(void);
+static void call_range(void);
+static void call_instance(void);
+static void stat_range(void);
+static void stat_instance(void);
+static void stat_periodic(void);
+static void user_range(void);
+static void user_instance(void);
 
 static node_t nodes[] = {
 	{"system.identity", &system_identity, "Identify server type and version", "string"},
@@ -61,6 +70,15 @@ static node_t nodes[] = {
 	{"system.methodHelp", &system_help, "Get help text for method", "string, string"},
 	{"system.methodSignature", &system_signature, "Get parameter signature for specified method", "array, string"},
 	{"system.status", &system_status, "Return server status information", "struct"},
+	{"server.status", &server_status, "Return server status string", "string"}, 
+	{"server.control", &server_control, "Return control request", "boolean, string"},
+	{"call.range", &call_range, "Return list of active calls", "array"},
+	{"call.instance", &call_instance, "Return specific call instance", "struct, string"},
+	{"stat.range", &stat_range, "Return list of call stat nodes", "array"},
+	{"stat.instance", &stat_instance, "return specific statistic node", "struct, string"},
+	{"stat.periodic", &stat_periodic, "return periodic statistics of node", "struct, string"},
+	{"user.range", &user_range, "Return list of user registrations", "array"},
+	{"user.instance", &user_instance, "Return specific user registration", "struct, string"},
 	{NULL, NULL, NULL, NULL}
 };
 
@@ -942,6 +960,400 @@ static void system_identity(void)
 	reply(buffer);
 }
 
+static bool iocontrol(const char *cmd)
+{
+	char buffer[512];
+	FILE *fp;
+#ifdef	_MSWINDOWS_
+	snprintf(buffer, sizeof(buffer), "%s\n", cmd);
+#else
+	snprintf(buffer, sizeof(buffer), "%d %s\n", getpid(), cmd);
+#endif
+	char *ep = strchr(buffer, '\n');
+	if(ep)
+		*(++ep) = 0;
+
+#ifndef	_MSWINDOWS_
+	int signo;
+	sigset_t sigs;
+	sigemptyset(&sigs);
+	sigaddset(&sigs, SIGUSR1);
+	sigaddset(&sigs, SIGUSR2);
+	sigaddset(&sigs, SIGALRM);
+	sigprocmask(SIG_BLOCK, &sigs, NULL);
+#endif	
+
+	fp = fopen(control_file, "w");
+	if(!fp)
+		fault(2, "Server Offline");
+
+	fputs(buffer, fp);
+	fclose(fp);
+#ifndef	_MSWINDOWS_
+	alarm(60);
+#ifdef	HAVE_SIGWAIT2
+	sigwait(&sigs, &signo);
+#else
+	signo = sigwait(&sigs);
+#endif
+	if(signo == SIGUSR2)
+		return false;
+	if(signo == SIGALRM)
+		fault(6, "Request Timed Out");
+#endif
+	return true;
+}
+
+static void server_control(void)
+{
+	char buffer[512];
+
+	if(params.argc != 1)
+		fault(3, "Invalid Parameters");
+
+	const char *command = getIndexed(1);
+	if(!command || !*command)
+		fault(5, "Invalid Command Argument");	
+
+	response(buffer, sizeof(buffer), "^(b)", iocontrol(command));
+	reply(buffer);
+}
+
+static void call_instance(void)
+{
+	mapped_view<MappedCall> cr(REGISTRY_MAP);
+	unsigned index = 0;
+	char id[32];
+	MappedCall copy;
+	char buffer[1024];
+	rpcint_t diff = 0;
+	time_t now;
+
+	if(params.argc != 1)
+		fault(3, "Invalid Parameters");
+
+	const char *cid = getIndexed(1);
+	if(!cid || !*cid)
+		fault(5, "Invalid Command Argument");	
+
+
+	unsigned count = cr.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	time(&now);
+	while(index < count) {
+		const MappedCall *map = const_cast<const MappedCall *>(cr(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(!map->created)
+			continue;
+
+		snprintf(id, sizeof(id), "%08x:%d", map->sequence, map->cid);
+		if(!String::equal(id, cid))
+			continue;
+	
+		if(map->active) {
+			time(&now);
+			diff = (rpcint_t)(now - map->active);
+		}
+
+		response(buffer, sizeof(buffer), "^(tsssssi)",
+			map->created, map->state + 1, map->authorized,
+			map->source, map->target, map->display, diff);
+		reply(buffer);
+	}
+	fault(6, "Unknown Call");
+}
+
+static void call_range(void)
+{
+	mapped_view<MappedCall> cr(REGISTRY_MAP);
+	unsigned size;
+	unsigned index = 0;
+	char id[32];
+	MappedCall copy;
+
+	if(params.argc != 0)
+		fault(3, "Invalid Parameters");
+
+	unsigned count = cr.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	size = count * 64 + 128;
+	char *buffer = (char *)malloc(size);
+	response(buffer, size, "^[");
+
+	while(index < count) {
+		const MappedCall *map = const_cast<const MappedCall *>(cr(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(!map->created)
+			continue;
+	
+		snprintf(id, sizeof(id), "%08x:%d", map->sequence, map->cid);
+		response(buffer, size, "!s", buffer);
+	}
+	response(buffer, size, "]");
+	reply(buffer);
+}
+
+static void stat_periodic(void)
+{
+	mapped_view<stats> sta(STAT_MAP);
+	unsigned index = 0;
+	stats copy;
+	char buffer[1024];
+
+	if(params.argc != 1)
+		fault(3, "Invalid Parameters");
+
+	const char *cid = getIndexed(1);
+	if(!cid || !*cid)
+		fault(5, "Invalid Command Argument");	
+
+	unsigned count = sta.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	while(index < count) {
+		const stats *map = const_cast<const stats *>(sta(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(!String::equal(map->id, cid))
+			continue;
+	
+		response(buffer, sizeof(buffer), "^(itiiiiii)",
+			(rpcint_t)map->limit, map->lastcall,
+			(rpcint_t)map->data[0].pperiod, (rpcint_t)map->data[0].pmin, (rpcint_t)map->data[0].pmax,
+			(rpcint_t)map->data[1].pperiod, (rpcint_t)map->data[1].pmin, (rpcint_t)map->data[1].pmax);
+		reply(buffer);
+	}
+	fault(7, "Unknown Stat");
+}
+
+static void stat_instance(void)
+{
+	mapped_view<stats> sta(STAT_MAP);
+	unsigned index = 0;
+	stats copy;
+	char buffer[1024];
+
+	if(params.argc != 1)
+		fault(3, "Invalid Parameters");
+
+	const char *cid = getIndexed(1);
+	if(!cid || !*cid)
+		fault(5, "Invalid Command Argument");	
+
+	unsigned count = sta.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	while(index < count) {
+		const stats *map = const_cast<const stats *>(sta(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(!String::equal(map->id, cid))
+			continue;
+	
+		response(buffer, sizeof(buffer), "^(itiiiiii)",
+			(rpcint_t)map->limit, map->lastcall,
+			(rpcint_t)map->data[0].total, (rpcint_t)map->data[0].current, (rpcint_t)map->data[0].peak,
+			(rpcint_t)map->data[1].total, (rpcint_t)map->data[1].current, (rpcint_t)map->data[1].peak);
+		reply(buffer);
+	}
+	fault(7, "Unknown Stat");
+}
+
+static void stat_range(void)
+{
+	mapped_view<stats> sta(STAT_MAP);
+	unsigned size;
+	unsigned index = 0;
+	stats copy;
+
+	if(params.argc != 0)
+		fault(3, "Invalid Parameters");
+
+	unsigned count = sta.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	size = count * 48 + 128;
+	char *buffer = (char *)malloc(size);
+	response(buffer, size, "^[");
+
+	while(index < count) {
+		const stats *map = const_cast<const stats *>(sta(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(!map->id[0])
+			continue;
+
+		response(buffer, size, "!s", map->id);
+	}
+	response(buffer, size, "]");
+	reply(buffer);
+}
+
+static void user_instance(void)
+{
+	mapped_view<MappedRegistry> reg(REGISTRY_MAP);
+	unsigned index = 0;
+	char ext[48];
+	MappedRegistry copy;
+	char buffer[2048];
+	time_t now;
+	const char *status = "idle";
+
+	if(params.argc != 1)
+		fault(3, "Invalid Parameters");
+
+	const char *id = getIndexed(1);
+	if(!id || !*id)
+		fault(5, "Invalid Command Argument");	
+
+	unsigned count = reg.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	time(&now);
+	while(index < count) {
+		const MappedRegistry *map = const_cast<const MappedRegistry *>(reg(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(map->type != MappedRegistry::USER && map->type != MappedRegistry::SERVICE) 
+			continue;
+
+		if(map->expires < now)
+			continue;
+
+		if(!String::equal(map->userid, id))
+			continue;
+	
+		if(map->ext)
+			snprintf(ext, sizeof(ext), "%u", map->ext);
+		else
+			String::set(ext, sizeof(ext), map->userid);
+
+		if(map->inuse)
+			status = "busy";
+		else
+			switch(map->status) {
+			case MappedRegistry::AWAY:
+				status = "away";
+				break;
+			case MappedRegistry::DND:
+				status = "dnd";
+				break;
+			case MappedRegistry::BUSY:
+				status = "busy";
+				break;
+			default:
+				break;
+			}
+
+		response(buffer, sizeof(buffer), "^(ssssii)",
+			ext, map->display, map->profile.id, status,
+			(rpcint_t)map->inuse, (rpcint_t)map->profile.level);
+		reply(buffer);
+	}
+	fault(8, "Unknown User");
+}
+
+static void user_range(void)
+{
+	mapped_view<MappedRegistry> reg(REGISTRY_MAP);
+	unsigned size;
+	unsigned index = 0;
+	MappedRegistry copy;
+
+	if(params.argc != 0)
+		fault(3, "Invalid Parameters");
+
+	unsigned count = reg.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	size = count * 48 + 128;
+	char *buffer = (char *)malloc(size);
+	response(buffer, size, "^[");
+	time_t now;
+	time(&now);
+
+	while(index < count) {
+		const MappedRegistry *map = const_cast<const MappedRegistry *>(reg(index++));
+	
+		do {
+			memcpy(&copy, map, sizeof(copy));
+		} while(memcmp(&copy, map, sizeof(copy)));
+		map = &copy;
+
+		if(map->type != MappedRegistry::USER && map->type != MappedRegistry::SERVICE) 
+			continue;
+
+		if(map->expires < now)
+			continue;
+
+		response(buffer, size, "!s", map->userid);
+	}
+	response(buffer, size, "]");
+	reply(buffer);
+}
+
+static void server_status(void)
+{
+	mapped_view<MappedCall> cr(REGISTRY_MAP);
+	char *cp;
+	unsigned index = 0;
+	volatile const MappedCall *map;
+
+	if(params.argc != 0)
+		fault(3, "Invalid Parameters");
+
+	unsigned count = cr.getCount();
+	if(!count)
+		fault(2, "Server Offline");
+
+	cp = (char *)malloc(count + 1);
+	cp[count] = 0;
+	memset(cp, ' ', count);
+	while(index < count) {
+		map = cr(index++);
+		if(map->state[0])
+			cp[index - 1] = map->state[0];
+	}
+	char *buffer = (char *)malloc(count + 512);
+	response(buffer, count + 512, "^(s)", cp);
+	reply(buffer);
+}
+	
 static void system_status(void)
 {
 	time_t now;
@@ -954,7 +1366,7 @@ static void system_status(void)
 		fault(3, "Invalid Parameters");
 
 	if(stat(control_file, &ino))
-		fault(2, "Server offline");
+		fault(2, "Server Offline");
 
 	while(nodes[count].method)
 		++count;
